import unittest

from argshold.core import ArgumentHolder, FrozenArgumentHolder


class TestArgumentHolder(unittest.TestCase):

    def test_initialization(self):
        holder = ArgumentHolder(1, 2, a=3, b=4)
        self.assertEqual(holder.args, [1, 2])
        self.assertEqual(holder.kwargs, {"a": 3, "b": 4})

    def test_set_args_and_kwargs(self):
        holder = ArgumentHolder()
        holder.args = [10, 20]
        holder.kwargs = {"x": 30, "y": 40}
        self.assertEqual(holder.args, [10, 20])
        self.assertEqual(holder.kwargs, {"x": 30, "y": 40})

    def test_equality(self):
        holder1 = ArgumentHolder(1, 2, a=3, b=4)
        holder2 = ArgumentHolder(1, 2, a=3, b=4)
        holder3 = ArgumentHolder(1, 2, a=5)
        self.assertEqual(holder1, holder2)
        self.assertNotEqual(holder1, holder3)

    def test_len(self):
        holder = ArgumentHolder(1, 2, a=3, b=4)
        self.assertEqual(len(holder), 4)

    def test_call(self):
        holder = ArgumentHolder(1, 2, a=3, b=4)

        def sample_function(*args, **kwargs):
            return args, kwargs

        result = holder.call(sample_function)
        self.assertEqual(result, ((1, 2), {"a": 3, "b": 4}))

    def test_copy(self):
        holder = ArgumentHolder(1, 2, a=3, b=4)
        copied = holder.copy()
        self.assertEqual(holder, copied)
        self.assertIsNot(holder, copied)


class TestFrozenArgumentHolder(unittest.TestCase):

    def test_initialization(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        self.assertEqual(holder.args, (1, 2))
        self.assertEqual(holder.kwargs, {"a": 3, "b": 4})

    def test_equality(self):
        holder1 = FrozenArgumentHolder(1, 2, a=3, b=4)
        holder2 = FrozenArgumentHolder(1, 2, a=3, b=4)
        holder3 = FrozenArgumentHolder(1, 2, a=5)
        self.assertEqual(holder1, holder2)
        self.assertNotEqual(holder1, holder3)

    def test_len(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        self.assertEqual(len(holder), 4)

    def test_hash(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        self.assertIsInstance(hash(holder), int)

    def test_call(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)

        def sample_function(*args, **kwargs):
            return args, kwargs

        result = holder.call(sample_function)
        self.assertEqual(result, ((1, 2), {"a": 3, "b": 4}))

    def test_frozen_behavior(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        with self.assertRaises(AttributeError):
            holder.args = [10, 20]
        with self.assertRaises(AttributeError):
            holder.kwargs = {"x": 30, "y": 40}


if __name__ == "__main__":
    unittest.main()



import functools
import unittest

from frozendict import frozendict

from argshold.core import FrozenArgumentHolder


class TestFrozenArgumentHolder(unittest.TestCase):

    def test_initialization(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        self.assertEqual(holder.args, (1, 2))
        self.assertEqual(holder.kwargs, frozendict(a=3, b=4))

    def test_equality(self):
        holder1 = FrozenArgumentHolder(1, 2, a=3, b=4)
        holder2 = FrozenArgumentHolder(1, 2, a=3, b=4)
        holder3 = FrozenArgumentHolder(1, 2, a=3)
        self.assertEqual(holder1, holder2)
        self.assertNotEqual(holder1, holder3)

    def test_hash(self):
        holder1 = FrozenArgumentHolder(1, 2, a=3, b=4)
        holder2 = FrozenArgumentHolder(1, 2, a=3, b=4)
        self.assertEqual(hash(holder1), hash(holder2))

    def test_len(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        self.assertEqual(len(holder), 4)

    def test_repr(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        self.assertEqual(repr(holder), "FrozenArgumentHolder(1, 2, a=3, b=4)")

    def test_immutable_attributes(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        with self.assertRaises(AttributeError):
            holder.args = (3, 4)
        with self.assertRaises(AttributeError):
            holder.kwargs = frozendict(c=5)

    def test_call(self):
        def sample_function(x, y, a=0, b=0):
            return x + y + a + b

        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        result = holder.call(sample_function)
        self.assertEqual(result, 10)

    def test_copy(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        copied = holder.copy()
        self.assertEqual(holder, copied)
        self.assertIsNot(holder, copied)

    def test_partialmethod(self):
        trueHolder = FrozenArgumentHolder(True)
        falseHolder = FrozenArgumentHolder(False)

        class Cell:
            def __init__(self):
                self._alive = False

            @property
            def alive(self):
                return self._alive

            def set_state(self, state):
                self._alive = bool(state)

            set_alive = trueHolder.partialmethod(set_state)
            set_dead = falseHolder.partialmethod(set_state)

        cell = Cell()
        self.assertFalse(cell.alive)
        cell.set_alive()
        self.assertTrue(cell.alive)
        cell.set_dead()
        self.assertFalse(cell.alive)


if __name__ == "__main__":
    unittest.main()



import unittest

from argshold.core import ArgumentHolder, FrozenArgumentHolder


class TestArgumentHolder(unittest.TestCase):
    def test_initialization(self):
        args = (1, 2, 3)
        kwargs = {"a": 10, "b": 20}
        holder = ArgumentHolder(*args, **kwargs)

        self.assertEqual(holder.args, list(args))
        self.assertEqual(holder.kwargs, kwargs)

    def test_equality(self):
        holder1 = ArgumentHolder(1, 2, a=3, b=4)
        holder2 = ArgumentHolder(1, 2, a=3, b=4)
        holder3 = ArgumentHolder(1, 2, a=3)

        self.assertEqual(holder1, holder2)
        self.assertNotEqual(holder1, holder3)

    def test_argument_modification(self):
        holder = ArgumentHolder(1, 2, 3, a=10, b=20)

        holder.args = [4, 5, 6]
        holder.kwargs = {"x": 30, "y": 40}

        self.assertEqual(holder.args, [4, 5, 6])
        self.assertEqual(holder.kwargs, {"x": 30, "y": 40})

    def test_len(self):
        holder = ArgumentHolder(1, 2, 3, a=10, b=20)
        self.assertEqual(len(holder), 5)

    def test_call(self):
        holder = ArgumentHolder(1, 2, a=3, b=4)
        result = holder.call(lambda x, y, a, b: x + y + a + b)
        self.assertEqual(result, 10)

    def test_copy(self):
        holder = ArgumentHolder(1, 2, a=3, b=4)
        copied_holder = holder.copy()

        self.assertEqual(copied_holder.args, holder.args)
        self.assertEqual(copied_holder.kwargs, holder.kwargs)
        self.assertIsNot(copied_holder, holder)

    def test_conversion(self):
        holder = ArgumentHolder(1, 2, a=3, b=4)
        frozen_holder = holder.toFrozenArgumentHolder()

        self.assertIsInstance(frozen_holder, FrozenArgumentHolder)
        self.assertEqual(frozen_holder.args, tuple(holder.args))
        self.assertEqual(frozen_holder.kwargs, holder.kwargs)


class TestFrozenArgumentHolder(unittest.TestCase):
    def test_initialization(self):
        args = (1, 2, 3)
        kwargs = {"a": 10, "b": 20}
        holder = FrozenArgumentHolder(*args, **kwargs)

        self.assertEqual(holder.args, args)
        self.assertEqual(holder.kwargs, kwargs)

    def test_equality(self):
        holder1 = FrozenArgumentHolder(1, 2, a=3, b=4)
        holder2 = FrozenArgumentHolder(1, 2, a=3, b=4)
        holder3 = FrozenArgumentHolder(1, 2, a=3)

        self.assertEqual(holder1, holder2)
        self.assertNotEqual(holder1, holder3)

    def test_len(self):
        holder = FrozenArgumentHolder(1, 2, 3, a=10, b=20)
        self.assertEqual(len(holder), 5)

    def test_call(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        result = holder.call(lambda x, y, a, b: x + y + a + b)
        self.assertEqual(result, 10)

    def test_hash(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        hash_value = hash(holder)

        self.assertIsInstance(hash_value, int)

    def test_conversion(self):
        holder = FrozenArgumentHolder(1, 2, a=3, b=4)
        normal_holder = holder.toArgumentHolder()

        self.assertIsInstance(normal_holder, ArgumentHolder)
        self.assertEqual(normal_holder.args, list(holder.args))
        self.assertEqual(normal_holder.kwargs, dict(holder.kwargs))


if __name__ == "__main__":
    unittest.main()



import unittest
from typing import Callable

from argshold.core import ArgumentHolder, FrozenArgumentHolder


class TestMatMulOperations(unittest.TestCase):

    def test_matmul_argument_holder(self):
        # Define a simple transformation function to test with matmul
        def multiply_by_two(x):
            return x * 2

        # Create an ArgumentHolder instance with some arguments
        arg_holder = ArgumentHolder(1, 2, 3, a=4, b=5)

        # Perform @matmul operation with the function `multiply_by_two`
        result = arg_holder @ multiply_by_two

        # Check if the positional arguments are transformed correctly
        self.assertEqual(result.args, [2, 4, 6])

        # Check if the keyword arguments are transformed correctly
        self.assertEqual(result.kwargs, {"a": 8, "b": 10})

    def test_matmul_frozen_argument_holder(self):
        # Define a simple transformation function to test with matmul
        def square(x):
            return x * x

        # Create a FrozenArgumentHolder instance with some arguments
        frozen_arg_holder = FrozenArgumentHolder(2, 3, 4, x=5, y=6)

        # Perform @matmul operation with the function `square`
        result = frozen_arg_holder @ square

        # Check if the positional arguments are transformed correctly
        self.assertEqual(list(result.args), [4, 9, 16])

        # Check if the keyword arguments are transformed correctly
        self.assertEqual(dict(result.kwargs), {"x": 25, "y": 36})

    def test_rmatmul_argument_holder(self):
        # Define a simple transformation function to test with rmatmul
        def add_five(x):
            return x + 5

        # Create an ArgumentHolder instance with some arguments
        arg_holder = ArgumentHolder(1, 2, 3, a=4, b=5)

        # Perform @matmul operation with the function `add_five`
        result = add_five @ arg_holder

        # Check if the positional arguments are transformed correctly
        self.assertEqual(result.args, [6, 7, 8])

        # Check if the keyword arguments are transformed correctly
        self.assertEqual(result.kwargs, {"a": 9, "b": 10})

    def test_rmatmul_frozen_argument_holder(self):
        # Define a simple transformation function to test with rmatmul
        def subtract_three(x):
            return x - 3

        # Create a FrozenArgumentHolder instance with some arguments
        frozen_arg_holder = FrozenArgumentHolder(10, 20, 30, x=40, y=50)

        # Perform @matmul operation with the function `subtract_three`
        result = subtract_three @ frozen_arg_holder

        # Check if the positional arguments are transformed correctly
        self.assertEqual(list(result.args), [7, 17, 27])

        # Check if the keyword arguments are transformed correctly
        self.assertEqual(dict(result.kwargs), {"x": 37, "y": 47})

    def test_imatmul_argument_holder(self):
        # Define a simple transformation function to test with imatmul
        def multiply_by_three(x):
            return x * 3

        # Create an ArgumentHolder instance with some arguments
        arg_holder = ArgumentHolder(1, 2, 3, a=4, b=5)

        # Perform @= operation with the function `multiply_by_three`
        arg_holder @= multiply_by_three

        # Check if the positional arguments are transformed correctly
        self.assertEqual(arg_holder.args, [3, 6, 9])

        # Check if the keyword arguments are transformed correctly
        self.assertEqual(arg_holder.kwargs, {"a": 12, "b": 15})

    def test_imatmul_frozen_argument_holder(self):
        # Define a simple transformation function to test with imatmul
        def add_ten(x):
            return x + 10

        # Create a FrozenArgumentHolder instance with some arguments
        frozen_arg_holder = FrozenArgumentHolder(1, 2, 3, x=4, y=5)

        # Perform @= operation with the function `add_ten`
        frozen_arg_holder @= add_ten

        # Check if the positional arguments are transformed correctly
        self.assertEqual(list(frozen_arg_holder.args), [11, 12, 13])

        # Check if the keyword arguments are transformed correctly
        self.assertEqual(dict(frozen_arg_holder.kwargs), {"x": 14, "y": 15})


if __name__ == "__main__":
    unittest.main()



import unittest

from frozendict import frozendict

from argshold.core import ArgumentHolder, FrozenArgumentHolder


class TestArgumentHolder(unittest.TestCase):
    def setUp(self):
        self.args = [1, 2, 3]
        self.kwargs = {"a": 10, "b": 20}
        self.holder = ArgumentHolder(*self.args, **self.kwargs)

    def test_initialization(self):
        self.assertEqual(self.holder.args, self.args)
        self.assertEqual(self.holder.kwargs, self.kwargs)

    def test_set_args(self):
        new_args = [4, 5, 6]
        self.holder.args = new_args
        self.assertEqual(self.holder.args, new_args)

    def test_set_kwargs(self):
        new_kwargs = {"x": 30, "y": 40}
        self.holder.kwargs = new_kwargs
        self.assertEqual(self.holder.kwargs, new_kwargs)

    def test_delete_args(self):
        del self.holder.args
        self.assertEqual(self.holder.args, [])

    def test_delete_kwargs(self):
        del self.holder.kwargs
        self.assertEqual(self.holder.kwargs, {})

    def test_copy(self):
        copy_holder = self.holder.copy()
        self.assertIsInstance(copy_holder, ArgumentHolder)
        self.assertEqual(copy_holder.args, self.args)
        self.assertEqual(copy_holder.kwargs, self.kwargs)

    def test_len(self):
        self.assertEqual(len(self.holder), len(self.args) + len(self.kwargs))


class TestFrozenArgumentHolder(unittest.TestCase):
    def setUp(self):
        self.args = (1, 2, 3)
        self.kwargs = frozendict({"a": 10, "b": 20})
        self.holder = FrozenArgumentHolder(*self.args, **self.kwargs)

    def test_initialization(self):
        self.assertEqual(self.holder.args, self.args)
        self.assertEqual(self.holder.kwargs, self.kwargs)

    def test_immutable_args(self):
        with self.assertRaises(AttributeError):
            self.holder.args = [4, 5, 6]

    def test_immutable_kwargs(self):
        with self.assertRaises(AttributeError):
            self.holder.kwargs = {"x": 30, "y": 40}

    def test_hash(self):
        self.assertEqual(hash(self.holder), hash((self.args, self.kwargs)))

    def test_copy(self):
        copy_holder = self.holder.copy()
        self.assertIsInstance(copy_holder, FrozenArgumentHolder)
        self.assertEqual(copy_holder.args, self.args)
        self.assertEqual(copy_holder.kwargs, self.kwargs)

    def test_len(self):
        self.assertEqual(len(self.holder), len(self.args) + len(self.kwargs))


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import abc
import functools
import types
from typing import *

from datarepr import datarepr
from frozendict import frozendict
from makeprop import makeprop
from unhash import unhash

__all__ = ["ArgumentHolder", "FrozenArgumentHolder"]


class BaseArgumentHolder(abc.ABC):

    __slots__ = ("_args", "_kwargs")

    @abc.abstractmethod
    def __eq__(self, other: Any, /) -> bool: ...

    @abc.abstractmethod
    def __hash__(self) -> int: ...

    @abc.abstractmethod
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

    def __len__(self) -> int:
        "This magic method implements len(self)."
        return len(self.args) + len(self.kwargs)

    def __matmul__(self, other: Callable) -> "BaseArgumentHolder":
        "This magic method implements self@other."
        args = [other(x) for x in self.args]
        kwargs = {k: other(v) for k, v in self.kwargs.items()}
        ans = type(self)(*args, **kwargs)
        return ans

    def __repr__(self) -> str:
        "This magic method implements repr(self)."
        return datarepr(type(self).__name__, *self.args, **self.kwargs)

    def __rmatmul__(self, other: Callable) -> "BaseArgumentHolder":
        "This magic method implements other@self."
        return self @ other

    @property
    @abc.abstractmethod
    def args(self): ...

    def call(self, callable: Callable, /) -> Any:
        "This method calls a callable using the arguments in the current instance."
        return callable(*self.args, **self.kwargs)

    def copy(self) -> "BaseArgumentHolder":
        "This method makes a copy of the current instance."
        return self.call(type(self))

    @property
    @abc.abstractmethod
    def kwargs(self): ...

    def partial(self, callable: Callable, /) -> functools.partial:
        "This method creates a functools.partial object."
        return functools.partial(callable, *self.args, **self.kwargs)

    def partialmethod(self, callable: Callable, /) -> functools.partial:
        "This method creates a functools.partialmethod object."
        return functools.partialmethod(
            callable,
            *self.args,
            **self.kwargs,
        )

    def toArgumentHolder(self) -> ArgumentHolder:
        "This method converts the current instance into a ArgumentHolder object."
        return self.call(ArgumentHolder)

    def toFrozenArgumentHolder(self) -> FrozenArgumentHolder:
        "This method converts the current instance into a FrozenArgumentHolder object."
        return self.call(FrozenArgumentHolder)


class ArgumentHolder(BaseArgumentHolder):

    def __eq__(self, other: Any, /) -> bool:
        "This magic method implements self==other."
        if not isinstance(other, ArgumentHolder):
            return False
        return (self.args, self.kwargs) == (other.args, other.kwargs)

    __hash__ = unhash

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        "This magic method sets up the current instance."
        self._args = list(args)
        self._kwargs = dict(kwargs)

    def __imatmul__(self, other: Callable) -> "ArgumentHolder":
        "This magic method implements self@=other."
        args0 = list(self.args)
        kwargs0 = dict(self.kwargs)
        args = [other(x) for x in self.args]
        kwargs = {k: other(v) for k, v in self.kwargs.items()}
        try:
            self.args = args
            self.kwargs = kwargs
        except BaseException as exc:
            self.args = args0
            self.kwargs = kwargs0
            raise
        else:
            return self

    @makeprop(delete=())
    def args(self, value: Iterable) -> None:
        "This property holds the positional arguments."
        value = list(value)
        self._args.clear()
        self._args.extend(value)
        return self._args

    @makeprop(delete=())
    def kwargs(self, value: Any) -> None:
        "This property holds the keyword arguments."
        value = dict(value)
        self._kwargs.clear()
        self._kwargs.update(value)
        return self._kwargs


class FrozenArgumentHolder(BaseArgumentHolder):

    def __eq__(self, other: Any, /) -> bool:
        "This magic method implements self==other."
        if not isinstance(other, FrozenArgumentHolder):
            return False
        return (self.args, self.kwargs) == (other.args, other.kwargs)

    def __hash__(self) -> int:
        "This magic method implements hash(self)."
        return (self.args, self.kwargs).__hash__()

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        "This magic method sets up the current instance."
        self._args = tuple(args)
        self._kwargs = frozendict(kwargs)

    @property
    def args(self) -> tuple:
        "This property holds the positional arguments."
        return self._args

    @property
    def kwargs(self) -> frozendict:
        "This property holds the keyword arguments."
        return self._kwargs
